@@ -17,7 +17,7 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-### setGeneric("plot")
+setGeneric("plot")
 
 setMethod("plot", signature(x="flexmix", y="missing"),
 function(x, y, mark=NULL, markcol="red",
